/****************************************************************/
/*																*/
/*				Bruch.cpp Version 1.3.5							*/
/*																*/
/*				Fachhochschule Jena - Informatik IIa			*/
/*				Projektarbeit Matrix							*/
/*				Set 2.08										*/
/*				Projektgruppe 1									*/
/*				Jrgen Dffinger  								*/
/*																*/
/****************************************************************/

/****************************************************************/
/*						Beschreibung:							*/
/*																*/
/*	Die Bruch.cpp enthlt den Code der Methoden der Klasse		*/
/*	Bruch. Die klasse Bruch ermglicht es Zahlen als Bruch zu	*/
/*	behandeln. Dabei werden arithmetische Operatoren und		*/
/*	Vergleichs-Operatoren zur Verfgung gestellt. Es werden nur */
/*	Ganzzahlausdrcke fr Zhler und Nenner verwendet werden,	*/
/*	da der Zhler und Nenner vom DatenTyp sind. Ein Krzungs-	*/
/*	algorithmus wurde ebenfalls implementiert. Dieser kann		*/
/*	nicht selber aufgerufen werden. Das Krzen wird automatisch	*/
/*	vorgenommen, sobald Werte des Bruchs verndert werden.		*/
/*																*/
/****************************************************************/



#include <iostream>
#include "Bruch.h"




using namespace std;



	//	Initialisierung
	void Bruch::init()
	{
		//	grte gemeinsame Teiler (ggT) bilden
		long ggt;
		ggt = ggT(zaehler, nenner);

		//	Bruch mithilfe des ggT krzen
		zaehler /= ggt;
		nenner /= ggt;

		setNenner();
	}



	
	//	Methode zur Bedingung Nenner nicht negativ
	void Bruch::setNenner() {if (nenner < 0) {nenner *= (-1); zaehler *= (-1);}}



	// grter gemeinsamer Teiler (ggT)
	Bruch::base_type Bruch::ggT(base_type a, base_type b) {return ((a % b) != 0) ? ggT(b, (a % b)) : b;}



	//	Bruch krzen
	Bruch::my_type Bruch::kuerzen() 
	{
		//	Deklaration des ggT mit entsprechender Berechnung
		Bruch::base_type ggt(ggT(zaehler, nenner));

		//	Deklaration des gekrzten Bruchs mit Zuweisung der gekrzten Werte
		Bruch::my_type tmp(zaehler / ggt, nenner / ggt);

		//	Bedingung Nenner nicht negativ prfen und entsprechend setzen
		tmp.setNenner();

		return tmp;
	}


	//	Konstruktor
	Bruch::Bruch()  : zaehler(0), nenner(1) {}



	//	Konstruktor zur bernahme eines Ganzzahl ausdrucks.
	Bruch::Bruch(base_type gz)  : zaehler(gz), nenner(1) {init();}



	//	Konstruktor mit bergebenen Vorgabewerten
	Bruch::Bruch(base_type z, base_type n) : zaehler(z), nenner(n) {init();}



	// Copy-Konstruktor
	Bruch::Bruch(const my_type& orig) : zaehler(orig.zaehler), nenner(orig.nenner) {init();}	



	//	Destruktor
	Bruch::~Bruch(){}



	//	Eingabe
	istream& operator >>(istream &is, Bruch::my_type &b)
	{
		//	Merken der alten Werte
		Bruch::my_type old(b.zaehler, b.nenner);

		//	Auslesen des Streams und in die Attribute schreiben
		is >> b.zaehler >> b.nenner;
		
		//	Prfen ob Nenner 0 ist
		if (b.nenner == 0)
		{
			b = old;	// Werte wieder zurcksetzen
			throw out_of_range("Nenner darf nicht 0 sein.");	// Exception auslsen
		}

		//	Bruch krzen
		b = b.kuerzen();

		//	Prfen ob Nenner negativ ist
		b.setNenner();

		return is;
	}



	//	Ausgabe
	ostream& operator <<(ostream &os, Bruch::my_type &b)
	{
		if (b.zaehler == 0)	
			os <<  "0"; 
		else if (b.nenner == 1)
			os << b.zaehler;
		else
			os << b.zaehler << "/" << b.nenner;
		return os;
	}



	//	Addition
	Bruch::my_type Bruch::operator + (const Bruch::my_type& b) const
	{return (Bruch::my_type ((b.nenner * zaehler) + (b.zaehler * nenner), b.nenner * nenner).kuerzen());}



	//	Addition mit Operator +=
	Bruch::my_type& Bruch::operator += (const Bruch::my_type& b) 
	{
		(*this) = (*this) + b;
		return (*this);
	}



	//	Subtraktion
	Bruch::my_type Bruch::operator - (const Bruch::my_type& b) const
	{return (Bruch::my_type((b.nenner * zaehler) - (b.zaehler * nenner), b.nenner * nenner).kuerzen());}



	//	Subtraktion mit Operator +=
	Bruch::my_type& Bruch::operator -= (const Bruch::my_type& b) 
	{
		(*this) = (*this) - b;
		return (*this);
	}



	// Multiplikation
	Bruch::my_type Bruch::operator * (const Bruch::my_type& b) const
	{return (Bruch::my_type(zaehler * b.zaehler, nenner * b.nenner).kuerzen());}



	// Multiplikation mit Operator *=
	Bruch::my_type& Bruch::operator *= (const Bruch::my_type& b)
	{
		(*this) = (*this) * b;
		return (*this);
	}



	//	Division 
	Bruch::my_type Bruch::operator / (const Bruch::my_type& b) const
	{
		//	Bedingung Division nicht durch 0 prfen (mathematischer Fehler)
		if (b.zaehler == 0) throw out_of_range("mathematischer Fehler -> Division durch 0");

		//	Deklaration
		Bruch::my_type tmp;

		//	Division durchfhren
		tmp.zaehler = zaehler * b.nenner;
		tmp.nenner = nenner * b.zaehler;

		tmp.kuerzen();
		return tmp;
	}



	//	Division mit operator /=
	Bruch::my_type& Bruch::operator /= (Bruch::my_type& b)
	{
		(*this) = (*this) / b;
		return b;
	}



	// unres Minus
	Bruch::my_type Bruch::operator - () {return (Bruch::my_type ((*this) * Bruch::my_type(-1,1)));}



	//	kleiner als
	bool Bruch::operator < (const Bruch::my_type& b) const
	{return (bool(((double)zaehler / (double)nenner) < ((double)b.zaehler / (double)b.nenner)));}



	//	grer als
	bool Bruch::operator > (const Bruch::my_type& b) const
	{return (bool(((double)zaehler / (double)nenner) > ((double)b.zaehler / (double)b.nenner)));}



	//	kleiner gleich
	bool Bruch::operator <= (const Bruch::my_type& b) const
	{return (bool(((double)zaehler / (double)nenner) <= ((double)b.zaehler / (double)b.nenner)));}



	//	grer gleich
	bool Bruch::operator >= (const Bruch::my_type& b) const
	{return (bool(((double)zaehler / (double)nenner) >= ((double)b.zaehler / (double)b.nenner)));}



	//	gleich
	bool Bruch::operator == (const Bruch::my_type& b) const
	{return (bool(((double)zaehler / (double)nenner) == ((double)b.zaehler / (double)b.nenner)));}



	//	ungleich
	bool Bruch::operator != (const Bruch::my_type& b) const
	{return (bool(((double)zaehler / (double)nenner) != ((double)b.zaehler / (double)b.nenner)));}


	
	//	Zuweisungsoperator
	Bruch::my_type Bruch::operator = (Bruch::my_type& b) 
	{
		zaehler = b.zaehler;
		nenner = b.nenner;
		return (*this);
	}



